#include "pop3.h"

static char buf_res[buf_length];
static char buf[buf_length];

char* sendCommand(SOCKET sock, const char* com, const char* errmsg, bool check_result) {
	sockSend(sock, com, errmsg);
	if(check_result) {
		int bytes_recv = recv(sock, buf_res, buf_length, 0);
		buf_res[bytes_recv] = 0;
		if(bytes_recv <= 0 || buf_res[0] != '+') {
			fputs(errmsg, stderr);
			exit(-1);
		}
		return buf_res;
	}
	return buf_res;
}

bool getType(const string& curline, string& result) {
	size_t twodot = curline.find(':');
	if(twodot == curline.npos) return false;
	istringstream curlinestream(curline.substr(0, twodot));
	string action;
	curlinestream >> action;
	size_t space = 0;
	for(; space<action.length() && action[space] == ' '; space++);
	for(size_t i = space; i<action.length(); i++) action[i] = tolower(action[i]);
	result = action.substr(space);
	return true;
}

string getDateFromMessage(SOCKET sock, const char* errmsg) {
	// get response
	int bytes_recv = recv(sock, buf, buf_length-1, 0);
	if(bytes_recv <= 0) {
		fputs(errmsg, stderr);
		printSocketError("recv");
		return string();
	}
	buf[bytes_recv] = 0;
	if(buf[0] != '+') {
		fputs(errmsg, stderr);
		return string();
	}
	string content(buf);
	int recv_message_len;
	sscanf(buf, "%*s %d", &recv_message_len);
	int recv_sum = bytes_recv;
	while(recv_sum < recv_message_len) {
		bytes_recv = recv(sock, buf, buf_length-1, 0);
		if(bytes_recv <= 0) {
			fputs(errmsg, stderr);
			printSocketError("recv");
			return string();
		}
		buf[bytes_recv] = 0;
		recv_sum += bytes_recv;
		content += string(buf);
	}
	// get received
	istringstream ss(content);
	string curline, action;
	getline(ss, curline);
	bool ok = false;
	while(!ss.eof()) {
		getline(ss, curline);
		if(!getType(curline, action)) continue;
		if(action != "received") continue;
		ok = true;
		break;
	}
	if(!ok) return string();
	// get date
	ok = false;
	string prevline;
	while(!ss.eof()) {
		prevline = curline;
		getline(ss, curline);
		if(curline.find(':') != curline.npos && curline.find(';') != curline.npos) {
			ok = true;
			break;
		}
	}
	if(!ok) return string();
	size_t lastfind = 0;
	do {
		size_t pos = curline.find(';', lastfind+1);
		if(pos == curline.npos) break;
		lastfind = pos;
	} while(lastfind < curline.length());
	return curline.substr(lastfind+1);
}

void auth(SOCKET sock, const char *user, const char *pass) {
	sprintf(buf, "USER %s\n", user);
	sendCommand(sock, buf, "authorization failed: invalid username");

	sprintf(buf, "PASS %s\n", pass);
	sendCommand(sock, buf, "authorization failed: invalid password");
}

void quit(SOCKET sock) {
	sprintf(buf, "QUIT\n");
	sendCommand(sock, buf, "ololo :)", false);
	closeSocket(sock);
}

int getLettersCount(SOCKET sock) {
	char *statres = sendCommand(sock, "STAT\n", "can't get exists mail");
	int letters_count;
	sscanf(statres, "%*s %d", &letters_count);
	return letters_count;
}